<?php
namespace Teknisa\EST\Factories;

use Doctrine\ORM\EntityManager;

class Ger01700UnidadeFactory {
    protected $connection;

    /**
     * Construtor da Factory.
     *
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }

    /**
     * Procedimento que executa a atualização dos dados
     * dos produtos para uma unidade de acordo com os
     * parâmetros recebidos.
     *
     * @param string $CDFILIAL
     * @param int $CDNVPRODUTO
     * @param char $IDCTRLESTQ
     * @param char $IDUTILLOTE
     */
    public function executeAtualizaProduto($CDFILIAL, $CDNVPRODUTO, $IDCTRLESTQ, $IDUTILLOTE) {
        $params                   = array();
        $params['P_CDFILIAL']     = $CDFILIAL;
        $params['P_CDNVPRODUTO']  = $CDNVPRODUTO;
        $params['P_IDCNTRESTOQ']  = $IDCTRLESTQ;
        $params['P_IDUTCTRLLOTE'] = $IDUTILLOTE;

        $sql = "INSERT INTO PRODFILI (CDFILIAL, CDPRODUTO, IDCNTRESTOQ, IDUTCTRLLOTE)
                    (SELECT :P_CDFILIAL, CDPRODUTO, :P_IDCNTRESTOQ, :P_IDUTCTRLLOTE
                       FROM PRODUTO
                      WHERE CDNVPRODUTO  = :P_CDNVPRODUTO
                        AND IDPRODATIVPR = 'S'
                        AND NOT EXISTS (SELECT :P_CDFILIAL, CDPRODUTO, :P_IDCNTRESTOQ, :P_IDUTCTRLLOTE
                                          FROM PRODFILI
                                         WHERE PRODFILI.CDFILIAL  = :P_CDFILIAL
                                           AND PRODFILI.CDPRODUTO = PRODUTO.CDPRODUTO))";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    /**
     * Função que retorna a parametrização de IDCTRLESTQ
     * de uma unidade.
     *
     * @param string $CDFILIAL
     * @param int $NRORG
     *
     * @return matrix
     */
    public function getIDCTRLESTQ($CDFILIAL, $NRORG) {
        $params               = array();
        $params['P_CDFILIAL'] = $CDFILIAL;
        $params['P_NRORG']    = $NRORG;

        $sql = "SELECT IDCTRLESTQ
                  FROM PARAMFILIAL
                 WHERE CDFILIAL = :P_CDFILIAL
                   AND NRORG    = :P_NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }

    /**
     * Função que retorna a parametrização de IDUTILLOTE
     * de uma unidade.
     *
     * @param string $CDFILIAL
     * @param int $NRORG
     *
     * @return matrix
     */
    public function getIDUTILLOTE($CDFILIAL, $NRORG) {
        $params               = array();
        $params['P_CDFILIAL'] = $CDFILIAL;
        $params['P_NRORG']    = $NRORG;

        $sql = "SELECT IDUTILLOTE
                  FROM PARAMFILIAL
                 WHERE CDFILIAL = :P_CDFILIAL
                   AND NRORG    = :P_NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }

    /**
     * Função que retorna a validação de endereço de uma
     * unidade.
     *
     * @param string $CDFILIAL
     * @param int $NRORG
     *
     * @return matrix
     */
    public function getValidationForEndefili($CDFILIAL, $NRORG) {
        $params               = array();
        $params['P_CDFILIAL'] = $CDFILIAL;
        $params['P_NRORG']    = $NRORG;

        $sql = "SELECT CDFILIAL,
                       listagg(IDTPENDEFILI, '.') WITHIN GROUP (ORDER BY NRORG,
                                                                         CDFILIAL,
                                                                         IDTPENDEFILI) AS CDROWS
                  FROM ENDEFILI
                 WHERE CDFILIAL = :P_CDFILIAL
                   AND NRORG    = :P_NRORG
              GROUP BY CDFILIAL,
                       NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }

    /**
     * Função que retorna se a parametrização exige os
     * campos opcionais de endereço ou não.
     *
     * @return matrix
     */
    public function getValidationForEndefiliFromParamgeral($NRORG) {
        $params            = array();
        $params['P_NRORG'] = $NRORG;

        $sql = "SELECT IDUTENDMODFISC
                  FROM PARAMGERAL
                 WHERE NRORG = :P_NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }

    public function getValidationForParamfilipadr($NRORG) {
        $params            = array();
        $params['P_NRORG'] = $NRORG;

        $sql = "SELECT DECODE(CDFILIPADRAO,
                              null, 'N',
                              'S') AS IDSHOWPARAMS,
                       NRORG
                  FROM PARAMGERAL
                 WHERE NRORG = :P_NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }

    /**
     * Função que retorna a validação da chave primária
     * de uma unidade.
     *
     * @param string $CDFILIAL
     * @param int $NRORG
     *
     * @return matrix
     */
    public function getValidationForUnidadePK($CDFILIAL, $NRORG) {
        $params               = array();
        $params['P_CDFILIAL'] = $CDFILIAL;
        $params['P_NRORG']    = $NRORG;

        $sql = "SELECT :P_CDFILIAL AS CDFILIAL,
                       :P_NRORG AS NRORG,
                       DECODE(count(*),
                              0, 'S',
                              'N') AS IDFREECODE
                  FROM FILIAL
                 WHERE CDFILIAL = :P_CDFILIAL
                   AND NRORG = :P_NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }
}